/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcgeneralconfigpage.h"

#include "../../widgets/kdclabel.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QDir>
#include <QToolButton>

#include <KLocale>
#include <KFileDialog>
#include <KUrl>

KDCGeneralConfigPage::KDCGeneralConfigPage( QWidget * parent ) 
	: QWidget( parent )
{
	kcfg_DefaultOutputPath = new KLineEdit;
	kcfg_DefaultOutputPath->setReadOnly( TRUE );
	QToolButton *button = new QToolButton;
	button->setText("...");
	kcfg_NiceLevel = new KIntSpinBox;
	kcfg_NiceLevel->setRange( -20, 19 );
	kcfg_BugReport = new QCheckBox( i18n("Show BugReport dialog if nesseserry\n( i.e. when apps crash )" ) );

	QGridLayout *gLayout = new QGridLayout;
	gLayout->addWidget( new KDCLabel( i18n("Default output path:") ), 0, 0 );
	gLayout->addWidget( kcfg_DefaultOutputPath, 0, 1 );
	gLayout->addWidget( button, 0, 2 );
	gLayout->addWidget( new KDCLabel( i18n("Nice Level:") ), 1, 0 );
	gLayout->addWidget( kcfg_NiceLevel, 1, 1 );

	QVBoxLayout *mainLayout = new QVBoxLayout;
	mainLayout->addLayout( gLayout );
	mainLayout->addWidget( kcfg_BugReport );

	mainLayout->addWidget( kcfg_BugReport );
	mainLayout->addStretch();

	setLayout( mainLayout );
	
	readConfig();

	connect( button, SIGNAL( clicked() ), this, SLOT( changeOutputPatch() ) );
}

KDCGeneralConfigPage::~KDCGeneralConfigPage()
{

}

void KDCGeneralConfigPage::readConfig()
{
	kcfg_DefaultOutputPath->setText( KDCConfig::defaultOutputPath() );
	kcfg_NiceLevel->setValue( KDCConfig::niceLevel() );
	if ( KDCConfig::bugReport() )
		kcfg_BugReport->setCheckState( Qt::Checked );
	else
		kcfg_BugReport->setCheckState( Qt::Unchecked );
}

void KDCGeneralConfigPage::saveSettings()
{
	KDCConfig::setDefaultOutputPath( kcfg_DefaultOutputPath->text() );
	KDCConfig::setNiceLevel( kcfg_NiceLevel->value() );
	if ( kcfg_BugReport->checkState() == Qt::Checked )
		KDCConfig::setBugReport( TRUE );
	else
		KDCConfig::setBugReport( FALSE );
}

void KDCGeneralConfigPage::changeOutputPatch()
{
	KUrl path = KFileDialog::getExistingDirectoryUrl( KUrl( kcfg_DefaultOutputPath->text() ), this, i18n("Select Default Output Directory") );
	if ( !path.isEmpty() )
		kcfg_DefaultOutputPath->setText( path.toLocalFile() );
}
