/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdclookandfeelconfigpage.h"

#include "../../kdcconfig.h"

#include <QLabel>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QVariant>

#include <KLocale>

KDCLookAndFeelConfigPage::KDCLookAndFeelConfigPage( QWidget * parent )
	: QWidget( parent )
{
	QVBoxLayout *mainLayout = new QVBoxLayout( this );
	kcfg_Animation = new QCheckBox( i18n("Animate disc structure ( hard for cpu )") );
	mainLayout->addWidget( kcfg_Animation );

	kcfg_UpdateGui = new KComboBox;
	kcfg_UpdateGui->addItem( "500 ms", QVariant( 500 ) );
	kcfg_UpdateGui->addItem( "1 s", QVariant( 1000 ) );
	kcfg_UpdateGui->addItem( "1.5 s", QVariant( 1500 ) );
	kcfg_UpdateGui->addItem( "2 s", QVariant( 2000 ) );
	kcfg_UpdateGui->addItem( "3 s", QVariant( 3000 ) );
	kcfg_UpdateGui->addItem( "5 s", QVariant( 5000 ) );
	kcfg_UpdateGui->addItem( "10 s", QVariant( 10000 ) );

	QHBoxLayout *hLayout1 = new QHBoxLayout;
	hLayout1->addWidget( new QLabel( i18n("GUI update interval:") ) );
	hLayout1->addWidget( kcfg_UpdateGui );

	kcfg_UpdateData = new KComboBox;
	kcfg_UpdateData->addItem( "500 ms", QVariant( 500 ) );
	kcfg_UpdateData->addItem( "1 s", QVariant( 1000 ) );
	kcfg_UpdateData->addItem( "1.5 s", QVariant( 1500 ) );
	kcfg_UpdateData->addItem( "2 s", QVariant( 2000 ) );
	kcfg_UpdateData->addItem( "3 s", QVariant( 3000 ) );

	QHBoxLayout *hLayout2 = new QHBoxLayout;
	hLayout2->addWidget( new QLabel( i18n("Data update interval:") ) );
	hLayout2->addWidget( kcfg_UpdateData );

	mainLayout->addLayout( hLayout1 );
	mainLayout->addLayout( hLayout2 );
	mainLayout->addStretch();
}

KDCLookAndFeelConfigPage::~KDCLookAndFeelConfigPage()
{

}

void KDCLookAndFeelConfigPage::readConfig()
{
	if ( KDCConfig::animation() )
		kcfg_Animation->setCheckState( Qt::Checked );
	else
		kcfg_Animation->setCheckState( Qt::Unchecked );
	kcfg_UpdateGui->setCurrentIndex( kcfg_UpdateGui->findData( KDCConfig::updateGui() ) );
	kcfg_UpdateData->setCurrentIndex( kcfg_UpdateData->findData( KDCConfig::updateData() ) );
}

void KDCLookAndFeelConfigPage::saveSettings()
{
	if ( kcfg_Animation->checkState() == Qt::Checked )
		KDCConfig::setAnimation( TRUE );
	else
		KDCConfig::setAnimation( FALSE );
	KDCConfig::setUpdateGui( kcfg_UpdateGui->itemData( kcfg_UpdateGui->currentIndex() ).toInt() );
	KDCConfig::setUpdateData( kcfg_UpdateData->itemData( kcfg_UpdateData->currentIndex() ).toInt() );
}
