/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcbuilddvddiscstructurejob.h"

#include "../kdcconfig.h"
#include "../kdcfilefactory.h"

#include <QByteArray>
#include <QFile>

#include <KLocale>

KDCBuildDVDDiscStructureJob::KDCBuildDVDDiscStructureJob( KDCDVDDisc *disc, QObject *parent )
	: KDCJob( parent )
{
	p_totalSum = 0;
	p_prevTotalSum = 0;
	p_prevSize = 0;
	p_disc = disc;

	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::MergedChannels );
	p_process->setWorkingDirectory( KDCFileFactory::discDirectory( disc ) );

	connect( p_process, SIGNAL( readyReadStandardOutput() ), this, SLOT( on_readyReadStandardOutput() ) );
	connect( p_process, SIGNAL( finished( int , QProcess::ExitStatus ) ), this, SLOT( on_finished( int , QProcess::ExitStatus ) ) );

	QStringList arguments;
	arguments << "-n" << QString::number( KDCConfig::niceLevel() );
	arguments << "dvdauthor" << "-x" << KDCFileFactory::discConfigFile( disc );

	p_process->setProgram( "nice", arguments );

	int frameCount = 0;
	QList<QTreeWidgetItem*> list = disc->takeMovies();
	QList<QTreeWidgetItem*>::const_iterator it;
	KDCMovie *movie;
	KDCMovieGroup *group;
	for ( it = list.constBegin(); it != list.constEnd(); ++it )
	{
		if ( ( *it )->type() == KDC::Movie )
		{
			movie = static_cast<KDCMovie*>( *it );
			//frameCount += movie->videoStream()->metaData()->frameCount();
			frameCount += qRound( movie->videoStream()->metaData()->length()/1000.0*movie->fps() );
			p_files.append( KDCFileFactory::movieName( movie ) );
		}
		else
		{
			group = static_cast<KDCMovieGroup*>( *it );
			frameCount += qRound( group->length()/1000.0*group->movie( 0 )->fps() );
			p_files.append( KDCFileFactory::movieName( group ) );
		}
	}

	setTotalSize( frameCount );
	qDebug( "KDCBuildDVDDiscStructureJob::frameCount = %d", frameCount );
}

KDCBuildDVDDiscStructureJob::~KDCBuildDVDDiscStructureJob()
{
	if ( p_process )
	{
		p_process->disconnect();
		if ( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
	qDebug( "~KDCBuildDVDDiscStructureJob" );
}

void KDCBuildDVDDiscStructureJob::start()
{
	QFile file;
	QStringList::const_iterator it;
	qint64 size = 0;
	for ( it = p_files.constBegin(); it != p_files.constEnd(); ++it )
	{
		file.setFileName( ( *it ) );
		size += file.size();
	}
	double tmp = 1 + 2*(double)size/( 1024*1024 );
	p_const = (double)totalSize()/tmp;
	qDebug( "Files Size = %f", p_const );
	emit jobMainProgram( "dvdauthor" );
	emit jobTotalSize( totalSize() );
	emit percent( 0 );
	emit infoMessage( i18n("Start building disc structure") );
	p_process->start();
	p_timerID = startTimer( KDCConfig::updateData() );
}

void KDCBuildDVDDiscStructureJob::on_readyReadStandardOutput()
{
/*
	QByteArray data = p_process->readAllStandardOutput();
	int idx = data.indexOf( 'a', 15 );
	if ( idx != -1 )
	{
		idx += 3;
		QByteArray tmp;
		char byte = data.at( idx );
		while ( byte != 'M' )
		{
			tmp.append( byte );
			++idx;
			byte = data.at( idx );
		}
		tmp.trimmed();
		int size = tmp.toInt();
		if ( p_prevSize <= size )
			p_totalSum += size - p_prevSize;
		else
			p_totalSum += size;
		emit progress( qRound( p_totalSum*p_const ) );
		emit speed( ( p_totalSum - p_prevTotalSum )*p_const );
		p_prevSize = size;
		p_prevTotalSum = p_totalSum;
	}
*/
}

void KDCBuildDVDDiscStructureJob::on_finished( int code, QProcess::ExitStatus status )
{
	killTimer( p_timerID );
	if ( status == QProcess::CrashExit )
	{
		setError( i18n("There was an error during process. Error code is %1").arg( code ) );
		KDCFileFactory::removeDirectory( KDCFileFactory::discOutputDirectory( p_disc ) + "/disc" );
	}
	else
	{
		emit infoMessage( i18n("Buildeed disc structure sucesfully finished") );
		emit percent( 100 );
		if ( p_disc->settings()->deleteTempFilesEnabled() )
		{
			QList<QTreeWidgetItem*> movies = p_disc->takeMovies();
			QList<QTreeWidgetItem*>::const_iterator it;
			for ( it = movies.constBegin(); it != movies.constEnd(); ++it )
				if ( ( *it )->type() == KDC::Movie )
					KDCFileFactory::removeFile( KDCFileFactory::movieName( static_cast<KDCMovie*>( *it ) ) );
				else
					KDCFileFactory::removeFile( KDCFileFactory::movieName( static_cast<KDCMovieGroup*>( *it ) ) );
		}
	}
	KDCFileFactory::removeFile( KDCFileFactory::discConfigFile( p_disc ) );

	qDebug( "KDCBuildDVDDiscStructureJob::end::emit finished()" );
	emit finished( this );
}

void KDCBuildDVDDiscStructureJob::timerEvent( QTimerEvent *event )
{
	if ( event->timerId() == p_timerID )
	{
		QByteArray data = p_process->readAllStandardOutput();
		int idx = data.indexOf( 'a', 15 );
		if ( idx != -1 )
		{
			idx += 3;
			QByteArray tmp;
			char byte = data.at( idx );
			while ( byte != 'M' )
			{
				tmp.append( byte );
				++idx;
				byte = data.at( idx );
			}
			tmp.trimmed();
			int size = tmp.toInt();
			if ( p_prevSize <= size )
				p_totalSum += size - p_prevSize;
			else
				p_totalSum += size;
			emit progress( qRound( p_totalSum*p_const ) );
			emit speed( ( p_totalSum - p_prevTotalSum )*p_const );
			p_prevSize = size;
			p_prevTotalSum = p_totalSum;
		}
	}
}
