/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcstaticpreviewjob.h"

#include "kdcmovieconvertjob.h"
#include "../kdcfilefactory.h"
#include "../kdcconfig.h"
#include "../widgets/kdcmplayerwidget.h"

#include <QStringList>
#include <QWidget>
#include <QHBoxLayout>
#include <QProgressBar>

#include <KLocale>
#include <KMessageBox>

KDCStaticPreviewJob::KDCStaticPreviewJob( const KDCMovie *movie, QObject *parent )
	: KDCCompositeJob( KDCCompositeJob::No, parent )
{
	KDCStaticPreviewJobPrivate( movie );
}

KDCStaticPreviewJob::KDCStaticPreviewJob( const KDCMovieGroup *group, QObject *parent )
	: KDCCompositeJob( KDCCompositeJob::No, parent )
{
	KDCStaticPreviewJobPrivate( group->movie( 0 ) );
}

void KDCStaticPreviewJob::KDCStaticPreviewJobPrivate( const KDCMovie *movie )
{
	p_tempFile = new KTemporaryFile;
	p_tempFile->setSuffix( ".mpg" );
	p_tempFile->open();

	p_movieFps = movie->fps();

	KDCMovieConvertJob *cJob = new KDCMovieConvertJob( movie, this );
	cJob->setOutputFile( p_tempFile->fileName() );
	connect( cJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_mencoderJobFinished( KDCJob * ) ) );
	connect( cJob, SIGNAL( progress( int ) ), this, SLOT( on_jobProgress( int ) ) );
	addSubjob( cJob );

	p_dlg = new KDCDialog( 0, Qt::WindowTitleHint );
	p_dlg->setWindowTitle( i18n("KDVDCreator - preview") );
	connect( p_dlg, SIGNAL( finished() ), this, SLOT( dialogClose() ) );

	KDCMPlayerWidget *p_mpl = new KDCMPlayerWidget;
	p_mpl->showControlButton( KDCMPlayerWidget::STOP, FALSE );
	p_mpl->showTimeSlider( FALSE );
	p_mpl->setWindowSize( 433, 244 );
	double aspect = 4.0/3.0;
	if ( movie->videoStream()->aspectRatio() == KDC::WideScreen )
		aspect = 16.0/9.0;
	p_mpl->setMovieProperties( movie->videoStream()->width(), movie->videoStream()->height(), aspect );

	QWidget *widget = p_mpl->controlPanel();
	QHBoxLayout *layout = static_cast<QHBoxLayout*>( widget->layout() );
	p_bar = new QProgressBar;
	p_bar->setMaximumHeight( 20 );
	p_bar->setFormat( i18n("Buffer level %p%") );
	layout->insertWidget( 3, p_bar );

	p_dlg->setCloseEnabled( TRUE );
	p_dlg->setMainWidget( p_mpl );

	mJob = new KDCMPlayerJob( this );
	addSubjob( mJob );
	mJob->setFile( p_tempFile->fileName() );
	mJob->setWindowID( p_mpl->windowID() );
	mJob->setVideoDriver( KDCConfig::driver() );

	connect( mJob, SIGNAL( progress( int ) ), p_mpl, SLOT( setTimeSeek( int ) ) );
	connect( mJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_mplayerJobFinished( KDCJob * ) ) );
	connect( p_mpl, SIGNAL( pauseClicked() ), mJob, SLOT( forcePause() ) );
	connect( p_mpl, SIGNAL( playClicked() ), this, SLOT( slotStart() ) );
}

KDCStaticPreviewJob::~KDCStaticPreviewJob()
{
	delete p_tempFile;
}

void KDCStaticPreviewJob::dialogClose()
{
	mJob->stop();
	removeSubjob( firstSubjob() );
	emit finished( this );
	deleteLater();
}

void KDCStaticPreviewJob::on_mencoderJobFinished( KDCJob *job )
{
	if ( job->hasErrors() )
	{
		KMessageBox::error( 0, job->error() );
		delete p_dlg;
		dialogClose();
	}
}

void KDCStaticPreviewJob::start()
{
	firstSubjob()->start();
	p_dlg->show();
}

void KDCStaticPreviewJob::on_jobProgress( int value )
{
	p_pos = qRound( value/p_movieFps );
	if ( value > 150 )
		setFlag( 1, TRUE );
	else
		p_bar->setValue( qBound( 0, qRound( 100 - 14.287*( 7 - p_pos ) ), 100 ) );

	if ( flag( 1 ) )
	{
		if ( flag( 2 ) )
		{
			int currPos = mJob->currentTime() + 5;
			p_bar->setValue( qBound( 0, (p_pos - currPos + 5)*20, 100 ) );
			if ( currPos > p_pos )
				mJob->pause();
			else
				mJob->unpause();
		}
		else
		{
			setFlag( 2, TRUE );
			mJob->start();
		}
	}
}

void KDCStaticPreviewJob::slotStart()
{
	if ( mJob->isRunning() )
		return;
	mJob->startAndSeek( qMax( 0, p_pos - 5 ) );
}

void KDCStaticPreviewJob::on_mplayerJobFinished( KDCJob *job )
{
	if ( job->hasErrors() )
		KMessageBox::error( 0, job->error() );
}
