/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcsubtitlecatjob.h"

#include "../kdcfilefactory.h"

#include <QStringList>

#include <KLocale>

KDCSubtitleCatJob::KDCSubtitleCatJob( const KDCMovieGroup *group, int id, QObject *parent )
	: KDCJob( parent )
{
	p_id = id + 1;

	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::MergedChannels );
	connect( p_process, SIGNAL( readyReadStandardOutput() ), this, SLOT( on_readyReadStandardOutput() ) );
	connect( p_process, SIGNAL( finished( int , QProcess::ExitStatus ) ), this, SLOT( on_finished( int , QProcess::ExitStatus ) ) );

	p_process->setStandardOutputFile( KDCFileFactory::movieSubtitleFile( group, p_id ) );
	p_process->setWorkingDirectory( KDCFileFactory::discDirectory( group->disc() ) );

	p_files.clear();
	for ( int i=0; i<group->movie( 0 )->subtitleStreamGroup()->count(); ++i )
		p_files.append( KDCFileFactory::movieSubtitleFile( group->movie( i ), p_id ) );

	p_process->setProgram( "cat", p_files );

	p_files.append( KDCFileFactory::movieSubtitleFile( group, p_id ) );
	setTotalSize( 0 );
}

KDCSubtitleCatJob::~KDCSubtitleCatJob()
{
	if ( p_process )
	{
		disconnect( p_process, 0, 0, 0 );
		if ( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
}

void KDCSubtitleCatJob::start()
{
	emit jobTotalSize( 0 );
	emit percent( 0 );
	emit infoMessage( i18n("Start concateanting subtitle streams ( %1 )").arg( p_id ) );
	p_process->start();
}

void KDCSubtitleCatJob::on_readyReadStandardOutput()
{
	QString error( p_process->readAllStandardOutput() );
	setFlag( 1, TRUE );
	setError( error );
	p_process->close();
}

void KDCSubtitleCatJob::on_finished( int code, QProcess::ExitStatus status )
{
	if ( status == QProcess::CrashExit && !flag( 1 ) )
	{
		setError( i18n("There was an error during process. Error code is %1" ).arg( code ) );
		KDCFileFactory::removeFile( p_files.last() );
	}
	else
		emit infoMessage( i18n("Concateante subtitle streams ( %1 ) sucesfully finished").arg( p_id ) );

	for ( int i=0; i<p_files.size() - 1; ++i )
		KDCFileFactory::removeFile( p_files.at( i ) );

	emit finished( this );
}
