/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcaudiostream.h"

#include "lib/kdclib.h"
#include "kdcdisc.h"
#include "kdcmovie.h"

#include <KIcon>
#include <KLocale>

KDCAudioStream::KDCAudioStream( KDCMovie *movie, KDCAudioStreamMetaData *metaData )
	: QObject( 0 )
{
	p_movie = movie;
	p_enabled = KDC::Enabled;
	p_metaData = metaData;
	p_mode = KDC::Resample;
	p_updates = KDC::Disabled;
	p_bitrate = 0;
	p_delay = 0;
	p_channels = 2;
}

void KDCAudioStream::copySettings( const KDCAudioStream *stream )
{
	p_enabled = stream->enabledMode();
	p_mode = stream->mode();
	p_updates = stream->updatesMode();
	p_bitrate = stream->bitrate();
	p_delay = stream->delay();
	p_channels = stream->channelCount();
	refresh();
}

KDCAudioStream::KDCAudioStream( KDCAudioStream *stream )
	: QObject( 0 )
{
	p_metaData = new KDCAudioStreamMetaData(	stream->metaData()->id(), stream->metaData()->codec(),
																						stream->metaData()->channelCount(), stream->metaData()->sampleRate(),
																						stream->metaData()->bitrate() );
	p_bitrate = stream->bitrate();
	p_delay = stream->delay();
	p_codec = stream->codec();
	p_mode = stream->mode();
	p_enabled = stream->enabledMode();
	p_updates = stream->updatesMode();
	p_channels = stream->channelCount();
}

KDCAudioStreamMetaData* KDCAudioStream::metaData() const
{
	return p_metaData;
}

void KDCAudioStream::setEnabledMode( KDC::Mode enabled )
{
	p_enabled = enabled;
}

KDC::Mode KDCAudioStream::enabledMode() const
{
	return p_enabled;
}

KDCAudioStream::~KDCAudioStream()
{
	if ( p_metaData )
		delete p_metaData;
}

void KDCAudioStream::setChannelCount( int count )
{
	if ( p_channels != count )
	{
		qDebug( "KDCAudioStream::setChannelCount( %d )", count );
		p_channels = count;
		if ( p_channels > 2 )
			p_codec = KDC::AC3;
		if ( p_mode == KDC::Copy && count != p_metaData->channelCount() )
			setMode( KDC::Resample );
		else
			refresh();
	}
}

int KDCAudioStream::channelCount() const
{
	return p_channels;
}

void KDCAudioStream::setBitrate( int bitrate )
{
	if ( p_bitrate != bitrate )
	{
		qDebug( "KDCAudioStream::setBitrate( %d )", bitrate );
		p_bitrate = bitrate;
		if ( p_mode == KDC::Copy && bitrate != p_metaData->bitrate() )
			setMode( KDC::Resample );
		else
			refresh();
	}
}

int KDCAudioStream::bitrate() const
{
	return p_bitrate;
}

int KDCAudioStream::delay() const
{
	return p_delay;
}

void KDCAudioStream::setDelay( int delay )
{
	if ( p_delay != delay )
	{
		p_delay = delay;
		refresh();
	}
}

void KDCAudioStream::setCodec( KDC::AudioCodec codec )
{
	if ( p_codec != codec )
	{
		qDebug( "KDCAudioStream::setCodec( %d )", codec );
		p_codec = codec;
		if ( p_codec == KDC::MP2 )
			p_channels = 2;
		KDC::DiscType discType = p_movie->disc()->discType();
		if ( discType == KDC::DVD && p_movie->videoFormat() != KDC::PAL && p_codec == KDC::MP2 )
		{
			p_movie->setUpdatesMode( KDC::Disabled );
			p_movie->setVideoFormat( KDC::PAL );
			p_movie->setUpdatesMode( KDC::Enabled );
		}
		if ( p_mode == KDC::Copy )
		{
			if (	( codec == KDC::MP2 && p_metaData->codec() != "80" ) ||
						( codec == KDC::AC3 && p_metaData->codec() != "8192" ) )
				setMode( KDC::Resample );
		}
		else
			refresh();
	}
}

KDC::AudioCodec KDCAudioStream::codec() const
{
	return p_codec;
}

KDC::StreamMode KDCAudioStream::mode() const
{
	return p_mode;
}

void KDCAudioStream::setMode( KDC::StreamMode mode )
{
	if ( p_mode != mode )
	{
		qDebug( "KDCAudioStream::setMode( %d )", mode );
		p_mode = mode;
		if ( p_mode == KDC::Copy )
		{
			p_bitrate = p_metaData->bitrate();
			p_channels = 2;
			if ( p_metaData->codec() == "8192" )
			{
				p_codec = KDC::AC3;
				p_channels = p_metaData->channelCount();
			}
			else
				p_codec = KDC::MP2;
		}
		else
		{
			if ( p_movie->disc()->discType() == KDC::DVD && p_movie->videoStream()->hqDVDMode() == KDC::Enabled )
			{
				p_codec = KDC::AC3;
				p_channels = p_metaData->channelCount();
			}
		}
		refresh();
	}
}

void KDCAudioStream::refresh()
{
	if ( p_updates == KDC::Enabled )
	{
		// display
		QStringList val;
		if ( p_codec == KDC::MP2 )
			val.append( "MP2" );
		else
			if ( p_codec == KDC::AC3 )
				val.append( "AC3" );
		else
			val.append( "Auto" );
		val.append( QString::number( p_channels ) );
		val.append( QString::number( p_bitrate ) );

		p_size = qRound( 0.000125*p_movie->videoStream()->metaData()->length()*p_bitrate );
		p_info = QString( "\ta\t" + val.join("\t") );

		qDebug( "KDCAudioStream::refresh()::emit settingsChanged()" );
		emit settingsChanged();
	}
}

quint64 KDCAudioStream::size() const
{
	return p_size;
}

void KDCAudioStream::setUpdatesMode( KDC::Mode mode )
{
	if ( mode != p_updates )
		p_updates = mode;
}

KDC::Mode KDCAudioStream::updatesMode() const
{
	return p_updates;
}

bool KDCAudioStream::operator!=( const KDCAudioStream& stream )
{
	if ( this && &stream )
	{
		if ( p_bitrate != stream.bitrate() )
			return TRUE;
		if ( p_delay != stream.delay() )
			return TRUE;
		if ( p_codec != stream.codec() )
			return TRUE;
		if ( p_channels != stream.channelCount() )
			return TRUE;
		if ( p_enabled != stream.enabledMode() )
			return TRUE;

		return FALSE;
	}
	else
		if ( ( this != NULL && &stream == NULL ) || ( this == NULL && &stream != NULL ) )
			return TRUE;

	return FALSE;
}

QString KDCAudioStream::info() const
{
	return p_info;
}

bool KDCAudioStream::isSVCDAudioStream( const KDCAudioStreamMetaData *stream )
{
	if ( stream->codec() == "80" && stream->sampleRate() == 44100 && KDCLib::isInRange( stream->bitrate(), 32, 384 ) )
		return TRUE;
	else
		return FALSE;
}

bool KDCAudioStream::isVCDAudioStream( const KDCAudioStreamMetaData *stream )
{
	if ( stream->codec() == "80" && stream->sampleRate() == 44100 && stream->bitrate() == 224 )
		return TRUE;
	else
		return FALSE;
}

bool KDCAudioStream::isCVDAudioStream( const KDCAudioStreamMetaData *stream )
{
	if ( stream->codec() == "80" && stream->sampleRate() == 44100 && KDCLib::isInRange( stream->bitrate(), 32, 384 ) )
		return TRUE;
	else
		return FALSE;
}

bool KDCAudioStream::isDVDPALAudioStream( const KDCAudioStreamMetaData *stream )
{
	if (	( stream->codec() == "8192" || stream->codec() == "80" ) && stream->sampleRate() == 48000 &&
				KDCLib::isInRange( stream->bitrate(), 32, 1536 ) )
		return TRUE;
	else
		return FALSE;
}

bool KDCAudioStream::isDVDNTSCAudioStream( const KDCAudioStreamMetaData *stream )
{
	if (	stream->codec() == "8192" && stream->sampleRate() == 48000 && KDCLib::isInRange( stream->bitrate(), 32, 1536 ) )
		return TRUE;
	else
		return FALSE;
}
