/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KDCAUDIOSTREAM_H
#define KDCAUDIOSTREAM_H

#include "kdcnamespace.h"
#include "kdcaudiostreammetadata.h"

#include <QObject>
#include <QString>

class KDCMovie;

class KDCAudioStream : public QObject
{
Q_OBJECT
public:
	KDCAudioStream( KDCMovie *movie, KDCAudioStreamMetaData *metaData );
	KDCAudioStream( KDCAudioStream *stream );
	~KDCAudioStream();
	void copySettings( const KDCAudioStream *stream );
	KDCAudioStreamMetaData *metaData() const;

	void setBitrate( int audioBitrate );
	void setDelay( int delay );
	void setCodec( KDC::AudioCodec codec );
	void setMode( KDC::StreamMode mode );
	void setChannelCount( int count );

	int channelCount() const;
	int bitrate() const;
	KDC::AudioCodec codec() const;
	int delay() const;
	KDC::StreamMode mode() const;

	quint64 size() const;
	void refresh();
	void setUpdatesMode( KDC::Mode mode );
	KDC::Mode updatesMode() const;

	void setEnabledMode( KDC::Mode mode );
	KDC::Mode enabledMode() const;
	QString info() const;

	bool operator!=( const KDCAudioStream& );

	static bool isVCDAudioStream( const KDCAudioStreamMetaData *stream );
	static bool isSVCDAudioStream( const KDCAudioStreamMetaData *stream );
	static bool isDVDPALAudioStream( const KDCAudioStreamMetaData *stream );
	static bool isDVDNTSCAudioStream( const KDCAudioStreamMetaData *stream );
	static bool isCVDAudioStream( const KDCAudioStreamMetaData *stream );
signals:
	void settingsChanged();
private:
	KDC::AudioCodec p_codec;
	int p_bitrate;
	int p_channels;
	KDC::StreamMode p_mode;
	int p_delay;
	KDCAudioStreamMetaData *p_metaData;
	quint64 p_size;
	KDC::Mode p_enabled;
	bool p_changes;
	KDC::Mode p_updates;
	KDCMovie *p_movie;
	QString p_info;
};
#endif
