/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KDCDISC_H
#define KDCDISC_H

#include "kdcnamespace.h"
#include "kdcmovie.h"
#include "kdcvmgmenu.h"
#include "kdcsubtitlestream.h"
#include "kdcmoviegroup.h"
#include "kdcdiscitemdelegate.h"
#include "kdcdiscsettings.h"

#include <QTreeWidget>
#include <QString>
#include <QList>
#include <QUrl>
#include <QKeyEvent>
#include <QMouseEvent>

class KDCDisc : public QTreeWidget
{
Q_OBJECT
public:
	KDCDisc( KDC::DiscType type, int id, QWidget *parent = 0 );
	~KDCDisc();
	KDC::DiscType discType() const;
	virtual QString name() const = 0;
	KDCDiscSettings* settings() const;

	int id() const;
	bool isMenuAvalible();

	KDCMovie* movie( int id );
	virtual void addMovie( KDCMovie *movie ) = 0;
	virtual void deleteMovie( KDCMovie *movie ) = 0;
	QTreeWidgetItem* currentMovie() const;
	QStringList codeMovies( KDCMovie *movie );
	QList<QTreeWidgetItem*> takeMovies();

	KDCMovieGroup* movieGroup( int id );
	virtual void insertMovieGroup( int index, KDCMovieGroup *group ) = 0;
	virtual void destroyMovieGroup( KDCMovieGroup *group ) = 0;
	virtual void removeMovieFromGroup( KDCMovie *movie ) = 0;
	virtual void deleteMovieGroup( KDCMovieGroup *group ) = 0;
	virtual void moveMovieToMovieGroup( KDCMovie *movie, KDCMovieGroup *group ) = 0;
	QStringList codeMovieGroups( KDCMovie *movie );

	virtual QStringList checkErrors() = 0;

	quint64 size() const;

	virtual void moveUp() = 0;
	virtual void moveDown() = 0;
	virtual void refresh();
	int maxSubtitleSreams() const;
	int maxAudioStreams() const;
signals:
	void settingsChanged();
	void dropedSomeFiles( const QStringList& files );
	void objectChanged( QTreeWidgetItem *current );
protected:
	virtual QList<QUrl> codeItem( QTreeWidgetItem *item ) = 0;
	QStringList mimeTypes() const;
	Qt::DropActions supportedDropActions() const;
	void drawRow( QPainter *painter, const QStyleOptionViewItem& option, const QModelIndex& index ) const;
	void drawBranches( QPainter *painter, const QRect& rect, const QModelIndex& index ) const;
	void keyPressEvent( QKeyEvent *event );
public slots:
	void computeNewSize();
private slots:
	void on_itemDoubleClicked( QTreeWidgetItem *, int );
	void on_currentItemChanged( QTreeWidgetItem *, QTreeWidgetItem * );
	void on_itemEntered( QTreeWidgetItem *, int );
protected:
	void mouseMoveEvent( QMouseEvent *event );
	virtual void addMoviePrivate( KDCMovie *movie ) = 0;
	KDC::DiscType p_discType;
	bool p_menu;
	quint64 p_size;
	
	int p_max_subtitleStreams;
	int p_max_audioStreams;
	bool p_drag;
	int p_id;
	int p_movieID;
private:
	void setCurrentObject( QTreeWidgetItem *item );
private:
	KDCDiscItemDelegate *p_itemDelegate;
	QTreeWidgetItem *p_currentMovie;
	QTreeWidgetItem *p_currentObject;
	KDCDiscSettings *p_settings;
	QTreeWidgetItem *p_objectMouse;
	QTreeWidgetItem *p_movieMouse;
	int p_key;
};
#endif
