/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcdvddisc.h"

#include "kdcfilefactory.h"
#include "lib/kdclib.h"

#include <QTreeWidgetItem>
#include <QString>
#include <QDrag>
#include <QUrl>
#include <QMimeData>

#include <KLocale>

KDCDVDDisc::KDCDVDDisc( int id, QWidget *parent ) 
	: KDCDisc( KDC::DVD, id, parent )
{
	addVTS();
	p_max_audioStreams = 8;
	p_max_subtitleStreams = 32;
}

KDCDVDDisc::~KDCDVDDisc()
{

}

KDCVTS* KDCDVDDisc::addVTS()
{
	qDebug( "KDCDVDDisc::addVTS()::begin" );
	KDCVTS *vts = new KDCVTS;
	addTopLevelItem( vts );
	setCurrentItem( vts );
	qDebug( "KDCDVDDisc::addVTS()::end" );
	return vts;
}

void KDCDVDDisc::deleteVTS( QTreeWidgetItem *vts )
{
	qDebug( "KDCDVDDisc::deleteVTS()::begin" );
	invisibleRootItem()->removeChild( vts );
	if ( topLevelItemCount() == 0 )
		addVTS();
	delete vts;
	qDebug( "KDCDVDDisc::deleteVTS()::end" );
}

int KDCDVDDisc::vtsCount() const
{
	if ( p_menu )
		return topLevelItemCount() - 1;
	else
		return topLevelItemCount();
}

KDCVTS* KDCDVDDisc::vts( QTreeWidgetItem *item ) const
{
	qDebug( "KDCDVDDisc::currentVTS()::begin::item = %d", KDCLib::isNull( item ) );
	QTreeWidgetItem *vts = NULL;

	if ( item->type() == KDC::VTS )
		vts = item;
	else
	{
		if ( item->type() == KDC::MovieInfo )
			item = item->parent();
		vts = item->parent();
	}

	qDebug( "KDCDVDDisc::currentVTS()::end::vts = %d", KDCLib::isNull( vts ) );
	return static_cast<KDCVTS*>( vts ); 
}

KDCVTS* KDCDVDDisc::vts( int id ) const
{
	int count = 0;
	QTreeWidgetItem *item;
	for( int i=0; i<topLevelItemCount(); ++i )
	{
		item = topLevelItem( i );
		if ( item->type() == KDC::VTS )
		{
			if ( count == id )
				return static_cast<KDCVTS*>( item );
			else
				++count;
		}
	}
        return static_cast<KDCVTS*>( item );
}

void KDCDVDDisc::addMoviePrivate( KDCMovie *movie )
{
	++p_movieID;
	movie->p_id = p_movieID;
	movie->p_disc = this;
	movie->p_chapterGroup = new KDCChapterGroup( movie );
	QTreeWidgetItem *item = currentItem();
	if ( item )
	{
		if ( item->type() == KDC::Movie || item->type() == KDC::MovieGroup )
		{
			int index = item->parent()->indexOfChild( item );
			item->parent()->insertChild( index, movie );
		}
		else
			if ( item->type() == KDC::VTS )
				item->addChild( movie );
	}
	else
		topLevelItem( topLevelItemCount() - 1 )->addChild( movie );
}

void KDCDVDDisc::addMovie( KDCMovie *movie )
{
	qDebug( "KDCDVDDisc::addMovie()::begin" );
	addMoviePrivate( movie );
	KDCVideoStreamMetaData *video = movie->videoStream()->metaData();
	KDCAudioStreamMetaData *audio;
	int i;
	KDCAudioStreamGroup *group = movie->audioStreamGroup();
	movie->videoStream()->setCodec( KDC::mpeg2 );
	if ( movie->videoFormat() == KDC::PAL ) //PAL
	{
		// Video
		if ( KDCVideoStream::isDVDPALVideoStream( video ) )
			movie->videoStream()->setMode( KDC::Copy );
		else
		{
			movie->videoStream()->setMode( KDC::Resample );
			movie->videoStream()->setBitrate( 2500 );
			movie->videoStream()->setResolution( 720, 576 );
			movie->videoStream()->setAspectRatio( KDC::WideScreen );
			movie->setVideoFormat( KDC::PAL );
		}
		// Audio
		if ( group )
		{
			for( i=0; i<group->count(); ++i )
			{
				audio = group->stream( i )->metaData();
				if ( KDCAudioStream::isDVDPALAudioStream( audio ) )
					group->stream( i )->setMode( KDC::Copy );
				else
				{
					if ( audio->channelCount() == 6 )
					{
						group->stream( i )->setCodec( KDC::AC3 );
						group->stream( i )->setChannelCount( 6 );
						group->stream( i )->setBitrate( 384 );
					}
					else
					{
						group->stream( i )->setChannelCount( 2 );
						if ( audio->codec() == "80" )
							group->stream( i )->setCodec( KDC::MP2 );
						else
							group->stream( i )->setCodec( KDC::AC3 );
						group->stream( i )->setBitrate( 128 );
					}
					group->stream( i )->setMode( KDC::Resample );
					group->stream( i )->setDelay( 0 );
				}
				group->stream( i )->setEnabledMode( KDC::Enabled );
			}
		}	
	}
	else //NTSC
	{
		// Video
		if ( KDCVideoStream::isDVDNTSCVideoStream( video ) )
			movie->videoStream()->setMode( KDC::Copy );
		else
		{
			movie->videoStream()->setMode( KDC::Resample );
			movie->videoStream()->setBitrate( 2500 );
			movie->videoStream()->setResolution( 720, 480 );
			movie->videoStream()->setAspectRatio( KDC::WideScreen );
			movie->setVideoFormat( KDC::NTSC );
		}
		// Audio
		if ( group )
		{
			for( i=0; i<group->count(); ++i )
			{
				audio = group->stream( i )->metaData();
				if ( KDCAudioStream::isDVDNTSCAudioStream( audio ) )
					group->stream( i )->setMode( KDC::Copy );
				else
				{
					if (	audio->channelCount() == 6 )
					{
						group->stream( i )->setCodec( KDC::AC3 );
						group->stream( i )->setBitrate( 384 );
					}
					else
					{
						group->stream( i )->setCodec( KDC::AC3 );
						group->stream( i )->setChannelCount( 2 );
						group->stream( i )->setBitrate( 128 );
					}
					group->stream( i )->setMode( KDC::Resample );
					group->stream( i )->setDelay( 0 );
				}
				group->stream( i )->setEnabledMode( KDC::Enabled );
			}
		}
	}
	connect( movie, SIGNAL( settingsChanged() ), this, SLOT( computeNewSize() ) );

	movie->videoStream()->setUpdatesMode( KDC::Enabled );
	movie->videoStream()->refresh();
	if ( group )
	{
		for( i=0; i<group->count(); ++i )
		{
			group->stream( i )->setUpdatesMode( KDC::Enabled );
			group->stream( i )->refresh();
		}
	}
	qDebug( "KDCDVDDisc::addMovie()::refreshing movie" );
	movie->setUpdatesMode( KDC::Enabled );
	movie->refresh();
	qDebug( "KDCDVDDisc::addMovie()::end" );
}

void KDCDVDDisc::insertMovieGroup( int index, KDCMovieGroup *group )
{
	qDebug( "KDCDVDDisc::insertMovieGroup()::begin::index = %d", index );
	vts( currentItem() )->insertChild( index, group );
	++p_movieID;
	group->p_id = p_movieID;
	group->p_disc = this;
	group->p_chapterGroup = new KDCChapterGroup( group );
	qDebug( "KDCDVDDisc::insertMovieGroup()::end" );
}

void KDCDVDDisc::deleteMovie( KDCMovie *movie )
{
	qDebug( "KDCDVDDisc::deleteMovie()::begin" );
	if ( movie->movieGroup() )
		removeMovieFromGroup( movie );
	else
	{
		QTreeWidgetItem *parent = movie->QTreeWidgetItem::parent();
		parent->removeChild( movie );
	}
	delete movie;
	qDebug( "KDCDVDDisc::deleteMovie()::end" );
}

void KDCDVDDisc::destroyMovieGroup( KDCMovieGroup *group )
{
	qDebug( "KDCDVDDisc::destroyMovieGroup()::begin" );
	QTreeWidgetItem *parent = group->QTreeWidgetItem::parent(); //vts
	int index = parent->indexOfChild( group );
	QList<QTreeWidgetItem*> movies = group->takeMovies();
	deleteMovieGroup( group );
	parent->insertChildren( index, movies );
	qDebug( "KDCDVDDisc::destroyMovieGroup()::end" );
}

void KDCDVDDisc::removeMovieFromGroup( KDCMovie *movie )
{
	qDebug( "KDCDVDDisc::removeMovieFromGroup()::begin" );
	KDCMovieGroup *group = movie->movieGroup();
	group->removeMovie( movie );
	if ( group->count() == 1 )
		destroyMovieGroup( group );
	qDebug( "KDCDVDDisc::removeMovieFromGroup()::end" );
}

void KDCDVDDisc::deleteMovieGroup( KDCMovieGroup *group )
{
	qDebug( "KDCDVDDisc::deleteMovieGroup()::begin" );
	group->QTreeWidgetItem::parent()->removeChild( group );
	delete group;
	qDebug( "KDCDVDDisc::deleteMovieGroup()::end" );
}

QStringList KDCDVDDisc::checkErrors()
{
	qDebug( "KDCDVDDisc::checkErrors()::begin" );
	QTreeWidgetItem *parent;
	QTreeWidgetItem *item;
	KDCMovie *movie;
	KDCMovieGroup *group;
	KDCVTS *vts;
	QList<KDCMovie*> movies;
	QStringList errors;
	QString str;
	int i, j, k;
	for( i=0; i<topLevelItemCount(); ++i )
	{
		parent = topLevelItem( i );
		if ( parent->childCount() == 0 )
			errors << i18n("Video Title Set is empty." );
		else
		{
			for( j=0; j<parent->childCount(); ++j )
			{
				item = parent->child( j );
				if ( item->type() == KDC::Movie )
					movies.append( static_cast<KDCMovie*>( item ) );
				else
					if ( item->type() == KDC::MovieGroup )
					{
						group = static_cast<KDCMovieGroup*>( item );
						movies.append( static_cast<KDCMovie*>( group->movie( 0 ) ) );
						if ( group->hasErrors() )
							errors << i18n("Movies in clip ") + group->outputName() + i18n(" have different parameters.");
					}
			}
			for ( j=0; j<movies.size(); ++j )
				for ( k=j+1; k<movies.size(); ++k )
					if ( *( movies.at( j ) ) != *( movies.at( k ) ) )
					{
						str.clear();
						str.append( i18n("Movies: ") );
						str.append( movies.at( j )->fileName() + i18n(" and ") + movies.at( k )->fileName() + " " );
						str.append( "have different parameters." );
						errors << str;
						break;
					}
		}
		movies.clear();
	}

	if ( !errors.isEmpty() )
		errors.prepend( i18n("Disc: ") + name() );

	qDebug( "KDCDVDDisc::checkErrors()::end" );
	return errors;
}

void KDCDVDDisc::moveUp()
{
	qDebug( "KDCDVDDisc::moveUp()::begin" );
	blockSignals( TRUE );
	QTreeWidgetItem *item = currentItem();
	bool expand = item->isExpanded();
	if ( item->type() != KDC::VTS )
	{
		QTreeWidgetItem *parent = item->parent();
		int index = parent->indexOfChild( item );
		parent->takeChild( index );
		parent->insertChild( index - 1, item ); 
	}
	else
	{
		int index = indexOfTopLevelItem( item );
		takeTopLevelItem( index );
		insertTopLevelItem( index - 1, item );
	}
	item->setExpanded( expand );
	blockSignals( FALSE );
	setCurrentItem( item );
	qDebug( "KDCDVDDisc::moveUp()::end" );
}

void KDCDVDDisc::moveDown()
{
	qDebug( "KDCDVDDisc::moveDown()::begin" );
	blockSignals( TRUE );
	QTreeWidgetItem *item = currentItem();
	bool expand = item->isExpanded();
	if ( item->type() != KDC::VTS )
	{
		QTreeWidgetItem *parent = item->parent();
		int index = parent->indexOfChild( item );
		parent->takeChild( index );
		parent->insertChild( index + 1, item );
	}
	else
	{
		int index = indexOfTopLevelItem( item );
		takeTopLevelItem( index );
		insertTopLevelItem( index + 1, item );
	}
	item->setExpanded( expand );
	blockSignals( FALSE );
	setCurrentItem( item );
	qDebug( "KDCDVDDisc::moveDown()::end" );
}

void KDCDVDDisc::moveMovieToMovieGroup( KDCMovie *movie, KDCMovieGroup *group )
{
	qDebug( "KDCDVDDisc::moveMovieToMovieGroup()::begin" );
	movie->QTreeWidgetItem::parent()->removeChild( movie );
	group->addMovie( movie );
	qDebug( "KDCDVDDisc::moveMovieToMovieGroup()::end" );
}

QString KDCDVDDisc::name() const
{
	return QString("DVD");
}

void KDCDVDDisc::mouseMoveEvent( QMouseEvent *event )
{
	KDCDisc::mouseMoveEvent( event );
	if ( !( event->buttons() & Qt::LeftButton ) )
		return;

	qDebug( "KDCDVDDisc::mouseMoveEvent()::currentItem = %d", KDCLib::isNull( currentItem() ) );
	if ( currentItem() )
	{
		QMimeData *mData = new QMimeData;
		p_drag = TRUE;
		QDrag *drag = new QDrag( this );
		mData->setUrls( codeItem( currentItem() ) );
		drag->setMimeData( mData );

		qDebug( "KDCDVDDisc::mouseMoveEvent()::start draging" );
		drag->start( Qt::CopyAction | Qt::MoveAction );
	}
}

QList<QUrl> KDCDVDDisc::codeItem( QTreeWidgetItem *item )
{
	qDebug( "KDCDVDDisc::codeItem( %d )::begin", KDCLib::isNull( item ) );
	QList<QUrl> urls;

	if ( item->type() == KDC::MovieInfo )
		item = item->parent();

	if ( item->type() == KDC::VTS )
	{
		urls.append( QUrl( QString::number( indexOfTopLevelItem( item ) ) ) ); //vts
		urls.append( QUrl( QString::number( item->isExpanded() ) ) );
	}
	else
		if ( item->type() == KDC::Movie || item->type() == KDC::MovieGroup )
		{
			urls.append( QUrl( QString::number( indexOfTopLevelItem( item->parent() ) ) ) ); //vts
			urls.append( QUrl( QString::number( item->isExpanded() ) ) );
			urls.append( QUrl( QString::number( item->parent()->indexOfChild( item ) ) ) ); //group or movie
			urls.append( QUrl( QString::number( item->parent()->isExpanded() ) ) );
		}

	qDebug( "KDCDVDDisc::codeItem()::end::size = %d", urls.size() );
	return urls;
}

bool KDCDVDDisc::dropMimeData( QTreeWidgetItem *parent, int index, const QMimeData *data, Qt::DropAction action )
{
	qDebug( "KDCDVDDisc::dropMimeData()::begin::parent = %d", KDCLib::isNull( parent ) );
	QList<QUrl> urls;
	urls = data->urls();
	bool res = TRUE;

	if ( parent )
		setCurrentItem( parent );
	if ( !p_drag )
	{
		QStringList files;
		foreach( QUrl url, urls )
			files.append( url.toLocalFile() );

		files.removeLast();
		qDebug( "KDCDVDDisc::dropMimeData()::emit dropedSomeFiles()" );
		emit dropedSomeFiles( files );
	}
	else
	{
		p_drag = FALSE;
		if ( !parent )
			res = FALSE;
		else
		{
			QList<QUrl> parentUrls = codeItem( parent );
			qDebug( "KDCDVDDisc::dropMimeData()::drag = %d, drop = %d", urls.size(), parentUrls.size() );
			if ( urls != parentUrls )
			{
				switch ( urls.size() )
				{
					case 2:
					{
						if ( parentUrls.size() == 2 )
						{
							blockSignals( TRUE );
							int index1 = urls.first().toString().toInt();
							QTreeWidgetItem *vts = topLevelItem( index1 );
							int index2 = parentUrls.first().toString().toInt();
							takeTopLevelItem( index1 );
							insertTopLevelItem( index2, vts );
							blockSignals( FALSE );
							setCurrentItem( vts );
							vts->setExpanded( urls.last().toString().toInt() );
						}
						else
							res = FALSE;
						break;
					}
					case 4:
					{
						blockSignals( TRUE );
						int index1 = urls.first().toString().toInt(); //vts
						QTreeWidgetItem *vts = topLevelItem( index1 );
						int index2 = urls.at( 2 ).toString().toInt(); //group or movie
						QTreeWidgetItem *dragItem = vts->child( index2 );
						switch ( parentUrls.size() )
						{
							case 2:
							{
								int index3 = parentUrls.first().toString().toInt();
								QTreeWidgetItem *newVts = topLevelItem( index3 );
								vts->removeChild( dragItem );
								newVts->addChild( dragItem );
								newVts->setExpanded( urls.at( 2 ).toString().toInt() );
								break;
							}
							case 4:
							{
								int index3 = parentUrls.first().toString().toInt(); //vts
								QTreeWidgetItem *newVts = topLevelItem( index3 );
								int index4 = parentUrls.at( 2 ).toString().toInt(); //parent
								vts->removeChild( dragItem );
								newVts->insertChild( index4, dragItem );
								newVts->setExpanded( urls.at( 2 ).toString().toInt() );
								break;
							}
						}
						blockSignals( FALSE );
						setCurrentItem( dragItem );
						dragItem->setExpanded( urls.last().toString().toInt() );
						break;
					}
				}
			}
			else
				res = FALSE;
		}
	}
	refreshAllVTS();
	qDebug( "KDCDVDDisc::dropMimeData()::end" );
	return res;
}

void KDCDVDDisc::allMovieInSeparateVTS()
{
	qDebug( "KDCDVDDisc::allMovieInSeparateVTS()::begin" );
	blockSignals( TRUE );
	KDCVTS *vts;
	QTreeWidgetItem *item;
	QTreeWidgetItem *parent;
	int j;
	int count;
	int temp;
	for ( int i=0; i<topLevelItemCount(); ++i )
	{
		parent = topLevelItem( i );
		count = parent->childCount();
		if ( count > 0 )
		{
			if ( parent->child( 0 )->type() == KDC::VMGMenu )
			{
				--count;
				temp = 1;
			}
			else
				temp = 0;
			if ( count > 1 )
			{
				for ( j=1+temp; j<count; ++j )
				{
					item = parent->child( 1+temp );
					parent->takeChild( 1+temp );
					vts = addVTS();
					vts->addChild( item );
				}
			}
		}
	}
	blockSignals( FALSE );
	refreshAllVTS();
	qDebug( "KDCDVDDisc::allMovieInSeparateVTS()::end" );
}

void KDCDVDDisc::refreshAllVTS()
{
	KDCVTS *vts;
	QTreeWidgetItem *item;
	for( int i=0; i<topLevelItemCount(); ++i )
	{
		item = topLevelItem( i );
		if ( item->type() == KDC::VTS )
		{
			vts = static_cast<KDCVTS*>( item );
			vts->refresh();
		}
	}
}

void KDCDVDDisc::refresh()
{
	refreshAllVTS();
	KDCDisc::refresh();
}
