/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcsubtitlestream.h"

#include "kdcconfig.h"
#include "kdcnamespace.h"

#include <KIcon>

KDCSubtitleStream::KDCSubtitleStream( const QString& fileName, double fps )
	: QTreeWidgetItem( KDC::Subtitles )
{
	p_fileName = fileName;

	//set default parameters
	p_codec = KDCConfig::codec();
	p_size = KDCConfig::size();
	p_fps = fps;
	setTextAlignment( 0, Qt::AlignVCenter );
	setTextAlignment( 1, Qt::AlignHCenter | Qt::AlignVCenter );
	setTextAlignment( 2, Qt::AlignHCenter | Qt::AlignVCenter );
	setTextAlignment( 3, Qt::AlignHCenter | Qt::AlignVCenter );
	refresh();
}

KDCSubtitleStream::KDCSubtitleStream( KDCSubtitleStream *subs )
	: QTreeWidgetItem( KDC::Subtitles )
{
	p_fileName = subs->fileName();
	p_size = subs->textSize();
	p_codec = subs->textCodec();
	p_fps = subs->fps();
	refresh();
}

KDCSubtitleStream::~KDCSubtitleStream()
{

}

void KDCSubtitleStream::refresh()
{
	setIcon( 0, KIcon("text-plain") );
	setText( 0, p_fileName );
	setText( 1, p_codec );
	setText( 2, QString::number( p_size ) );
	setText( 3, QString::number( p_fps ) );
}

double KDCSubtitleStream::fps() const
{
	return p_fps;
}

QString KDCSubtitleStream::textCodec() const
{
	return p_codec;
}

int KDCSubtitleStream::textSize() const
{
	return p_size;
}
	
void KDCSubtitleStream::setFps( double fps )
{
	if ( p_fps != fps )
	{
		p_fps = fps;
		refresh();
	}
}

void KDCSubtitleStream::setTextCodec( const QString& codec )
{
	if ( p_codec != codec )
	{
		p_codec = codec;
		refresh();
	}
}

void KDCSubtitleStream::setTextSize( int size )
{
	if ( p_size != size )
	{
		p_size = size;
		refresh();
	}
}

QString KDCSubtitleStream::fileName() const
{
	return p_fileName;
}
