/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KDCVIDEOSTREAM_H
#define KDCVIDEOSTREAM_H

#include "kdcnamespace.h"
#include "kdcvideostreammetadata.h"

#include <QObject>

class KDCMovie;

class KDCVideoStream : public QObject
{
Q_OBJECT
public:
	KDCVideoStream( KDCMovie *movie, KDCVideoStreamMetaData *metaData );
	KDCVideoStream( KDCVideoStream *stream );
	~KDCVideoStream();
	void copySettings( const KDCVideoStream *stream );
	KDCVideoStreamMetaData *metaData() const;

	void setBitrate( int bitrate );
	void setResolution( int width, int height );
	void setMacroBlockMode( KDC::MacroBlockMode mode );
	void setDeinterlaceMode( KDC::DeinterlaceMode mode );
	void setTrellisMode( KDC::Mode mode );
	void setMode( KDC::StreamMode mode );
	void setAspectRatio( KDC::AspectRatio aspect );
	void setHQDVDMode( KDC::Mode mode );
	void setClosestGOPMode( KDC::Mode mode );
	void setCodec( KDC::VideoCodec codec );

	KDC::VideoCodec codec() const;
	KDC::Mode closestGOPMode() const;
	KDC::Mode hqDVDMode() const;
	KDC::AspectRatio aspectRatio() const;
	int bitrate() const;
	int width() const;
	int height() const;
	KDC::MacroBlockMode macroBlockMode() const;
	KDC::DeinterlaceMode deinterlaceMode() const;
	KDC::Mode trellisMode() const;	
	KDC::StreamMode mode() const;

	quint64 size() const;
	void refresh();
	void setUpdatesMode( KDC::Mode mode );
	KDC::Mode updatesMode() const;
	QString info() const;

	bool operator!=( const KDCVideoStream& );

	static bool isSVCDPALVideoStream( const KDCVideoStreamMetaData *stream );
	static bool isSVCDNTSCVideoStream( const KDCVideoStreamMetaData *stream );
	static bool isVCDPALVideoStream( const KDCVideoStreamMetaData *stream );
	static bool isVCDNTSCVideoStream( const KDCVideoStreamMetaData *stream );
	static bool isDVDPALVideoStream( const KDCVideoStreamMetaData *stream );
	static bool isDVDNTSCVideoStream( const KDCVideoStreamMetaData *stream );
	static bool isCVDPALVideoStream( const KDCVideoStreamMetaData *stream );
	static bool isCVDNTSCVideoStream( const KDCVideoStreamMetaData *stream );
	static bool isPALDVDResolution( int width, int height );
	static bool isNTSCDVDResolution( int width, int height );
signals:
	void settingsChanged();
private:
	KDC::AspectRatio p_ar;
	KDC::MacroBlockMode p_macroBlockMode;
	KDC::DeinterlaceMode p_deinterlaceMode;
	KDC::Mode p_trellisMode;
	int p_bitrate;
	KDC::StreamMode p_mode;
	int p_width;
	int p_height;
	KDCVideoStreamMetaData *p_metaData;
	quint64 p_size;
	KDC::Mode p_hq;
	KDC::Mode p_gop;
	KDC::VideoCodec p_codec;
	KDC::Mode p_updates;
	KDCMovie *p_movie;
	QString p_info;
};
#endif
