/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcresolutioncombobox.h"

#include "../../kdcdisc.h"

#include <QStringList>

KDCResolutionComboBox::KDCResolutionComboBox( const KDCMovie *movie, QWidget *parent )
	: KComboBox( parent )
{
	QStringList items;
	if ( movie->disc()->discType() == KDC::DVD && movie->videoStream()->codec() == KDC::mpeg2 )
	{
		if ( movie->videoFormat() == KDC::PAL )
			items << "720x576" << "704x576" << "352x576" << "352x288";
		else
			items << "720x480" << "704x480" << "352x480" << "352x240";
	}
	else
		if ( movie->disc()->discType() == KDC::DVD && movie->videoStream()->codec() == KDC::mpeg1 )
		{
			if ( movie->videoFormat() == KDC::PAL )
				items << "352x288";
			else
				items << "352x240";
		}
	else
		if ( movie->disc()->discType() == KDC::SVCD )
		{
			if ( movie->videoFormat() == KDC::PAL )
				items << "480x576";
			else
				items << "480x480";
		}
	else
		if ( movie->disc()->discType() == KDC::CVD )
		{
			if ( movie->videoFormat() == KDC::PAL )
				items << "352x576";
			else
				items << "352x480";
		}
	else
	{
		if ( movie->videoFormat() == KDC::PAL )
			items << "352x288";
		else
			items << "352x240";
	}

	addItems( items );
	QString str( QString::number( movie->videoStream()->width() ) + "x" + QString::number( movie->videoStream()->height() ) );
	setCurrentIndex( findText( str ) );
	connect( this, SIGNAL( currentIndexChanged( int ) ), this, SLOT( on_currentIndexChanged( int ) ) );
}

KDCResolutionComboBox::~KDCResolutionComboBox()
{

}

int KDCResolutionComboBox::width() const
{
	QString str = currentText();
	if ( !str.isEmpty() )
		return str.split( "x" ).at( 0 ).toInt();
	else
		return 0;
}

int KDCResolutionComboBox::height() const
{
	QString str = currentText();
	if ( !str.isEmpty() )
		return str.split( "x" ).at( 1 ).toInt();
	else
		return 0;
}

void KDCResolutionComboBox::on_currentIndexChanged( int )
{
	emit currentResolutionChanged( width(), height() );
}
