/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KDCMPLAYERWIDGET_H
#define KDCMPLAYERWIDGET_H

#include <QResizeEvent>
#include <QSlider>
#include <QToolButton>
#include <QContextMenuEvent>
#include <QList>
#include <QWidget>
#include <QLabel>
#include <QMap>
#include <KAction>

#include <KMenu>
#include <KSelectAction>

class KDCMPlayerWidget: public QWidget
{
Q_OBJECT
public:
	enum ControlButton { PLAY, STOP, PAUSE };
	KDCMPlayerWidget( QWidget *parent = 0 );
	~KDCMPlayerWidget();
	void setWindowSize( int width, int height );
	void setMovieProperties( int width, int height, double aspect, int length = 0 );
	void showControlButton( ControlButton button, bool show );
	QWidget* controlPanel() const;
	void showTimeSlider( bool );
	void showTimeLabel( bool );
	WId windowID() const;
	QString videoDriver();
	int audioID();
	void setAudioTracks( const QList<int>& );
	int currentPos() const;
public slots:
	void setTimeSeek( int value );
signals:
	void playClicked();
	void stopClicked();
	void pauseClicked();
	void timeSeekChanged( int );
	void videoDriverChanged( const QString& );
	void audioIDChanged( int );
private slots:
	void audioIDTriggered( QAction * );
protected:
	void resizeEvent( QResizeEvent *event );
	void contextMenuEvent( QContextMenuEvent *event );
private:
	int p_width;
	int p_height;
	double p_aspect;
	KMenu *p_popupMenu;
	KSelectAction *p_vo;
	KSelectAction *p_aid;
	QLabel *p_time;
	QSlider *p_timeSeek;

	QMap<ControlButton,QToolButton*> p_buttons;
	QWidget *p_panel;
	QWidget *p_mainWindow;
	QWidget *p_subWindow;
};
#endif
